import requests, urllib3, xmltodict, json
from httpx import AsyncClient
from dataclasses import dataclass
from xml.etree import ElementTree
from UPS_SDK.models import AccessRequest, TrackRequest, TrackResponse, TrackError
urllib3.disable_warnings()

@dataclass
class UPSApi:
    access_request: AccessRequest
    host: str = "https://onlinetools.ups.com/ups.app/xml/Track"
    
    @staticmethod
    def __create_response(text: str) -> TrackResponse:
        xpars = xmltodict.parse(text, dict_constructor=dict)
        track_response: dict = json.loads(json.dumps(xpars))["TrackResponse"]
        
        if "Error" in track_response["Response"].keys():
            raise TrackError(**track_response["Response"]["Error"])
        
        response = TrackResponse(**track_response)
        
        return response
    
    def prepare_request_data(self, track_request: TrackRequest) -> str:
        xml_data = ""
        xml_data += ElementTree.tostring(self.access_request.xml()).decode("utf-8")
        xml_data += ElementTree.tostring(track_request.xml()).decode("utf-8")
        return xml_data
    
    def get_package(self, track_request: TrackRequest):
        response = requests.post(self.host, self.prepare_request_data(track_request), verify=False)
        return self.__create_response(response.text)

    async def async_get_package(self, async_client: AsyncClient, track_request: TrackRequest):
        response = await async_client.post(self.host, data=self.prepare_request_data(track_request))
        return self.__create_response(response.text)


from __future__ import annotations
from pydantic import BaseModel
from UPS_SDK.models.TransactionReference import TransactionReference

class Response(BaseModel):
    TransactionReference: TransactionReference
    ResponseStatusCode: str
    ResponseStatusDescription: str


from __future__ import annotations
from pydantic import BaseModel, Field
from UPS_SDK.models.ActivityItem import ActivityItem
from UPS_SDK.models.Message import Message
from UPS_SDK.models.PackageWeight import PackageWeight
from UPS_SDK.models.ReferenceNumber import ReferenceNumber
from typing import List, Optional

class Package(BaseModel):
    TrackingNumber: str
    DeliveryIndicator: str
    RescheduledDeliveryDate: Optional[str]
    AllActivity: Optional[List[ActivityItem]] = Field(None, alias="Activity")

    Message: Optional[Message]
    PackageWeight: PackageWeight
    ReferenceNumber: Optional[ReferenceNumber]
    
    @property
    def get_US_activities(self) -> List[ActivityItem]:
        activities = []
        for activity in self.AllActivity:
            if activity.ActivityLocation is not None and activity.ActivityLocation.Address is not None and activity.ActivityLocation.Address.CountryCode == "US":
                activities.append(activity)
        return activities
    
    @property
    def is_delivered(self) -> bool:
        if "Delivered" in self.AllActivity[0].description:
            return True
        return False
    


from unittest import TestCase
import unittest
from main import UPSApi
from UPS_SDK.models import TrackResponse
from UPS_SDK.models import TrackError
from UPS_SDK.models import TrackType
from models.AccessRequest import AccessRequest
from models.TrackRequest import TrackRequest

class GetPackageTest(TestCase):
    WRONG_TRACKING_NUMBER = "1Z89RA79045600675612"
    CORRECT_TRACKING_NUMBER = "1Z89RA790456006756"
    
    def test_correct_package(self):
        correct_request = TrackRequest(
            track_type=TrackType.ByTrackingNumber, reference_number_or_tracking_number=self.CORRECT_TRACKING_NUMBER
        )
        correct_package = api.get_package(track_request=correct_request)
        self.assertIsInstance(correct_package, TrackResponse, "Correct Package Is Instance TrackResponse Object")
        
    def test_wrong_package(self):
        wrong_request = TrackRequest(
            track_type=TrackType.ByTrackingNumber, reference_number_or_tracking_number=self.WRONG_TRACKING_NUMBER
        )
        self.assertRaises(TrackError, api.get_package, track_request=wrong_request)
    
ACCESS_LICENSE_NUMBER = "0D9B373178F57052"
USER_ID = "BUZV"
PASSWORD = "Falkel.2021"
api = UPSApi(access_request=AccessRequest(access_license_number=ACCESS_LICENSE_NUMBER, user_id=USER_ID, password=PASSWORD))
unittest.main()


from setuptools import setup, find_packages

# The version of this tool is based on the following steps:
# https://packaging.python.org/guides/single-sourcing-package-version/
VERSION = {}

with open("./UPS_SDK/__init__.py") as fp:
    # pylint: disable=W0122
    exec(fp.read(), VERSION)

setup(
    name="ups_sdk",
    author="Esat Yılmaz",
    author_email="esatyilmaz3500@gmail.com",
    description="UPS Sdk",
    version=VERSION.get("__version__", "1.0.0"),
    packages=find_packages(where=".", exclude=["tests"]),
    classifiers=[
        "Development Status :: 1 - Planning",
        "Topic :: Utilities",
        "Programming Language :: Python :: 3.9",
    ],
)



from pydantic import BaseModel
from xml.etree import ElementTree as ET

class AccessRequest(BaseModel):
    access_license_number: str
    user_id: str
    password: str
    
    def xml(self) -> ET.Element:
        root = ET.Element("AccessRequest")
        ET.SubElement(root, "AccessLicenseNumber").text = self.access_license_number
        ET.SubElement(root, "UserId").text = self.user_id
        ET.SubElement(root, "Password").text = self.password
        return root
